@@ -1,7 +1,7 @@
 /*
  * gauche-math3d.c - 3D vector and matrix arithmetics
  *
- *  Copyright(C) 2002-2003 by Shiro Kawai (shiro@acm.org)
+ *  Copyright (c) 2002-2008  Shiro Kawai  <shiro@acm.org>
  *
  *  Permission to use, copy, modify, distribute this software and
  *  accompanying documentation for any purpose is hereby granted,
@@ -12,7 +12,7 @@
  *  warranty.  In no circumstances the author(s) shall be liable
  *  for any damages arising out of the use of this software.
  *
- *  $Id: gauche-math3d.c,v 1.25 2005/08/02 04:21:48 shirok Exp $
+ *  $Id: gauche-math3d.c,v 1.29 2008-06-05 17:33:04 shirok Exp $
  */
 
 #include <math.h>
@@ -1223,6 +1223,18 @@ void Scm_Matrix4fToQuatfv(float q[], const float m[])
     }
 }
 
+/*
+ * Transform a vector/point by quaternion
+ * calculates qvq*
+ */
+void Scm_QuatfTransformv(float r[], const float q[], const float v[])
+{
+    float qconj[4], s[4];
+    SCM_QUATF_CONJUGATEV(qconj, q);
+    Scm_QuatfMulv(s, q, v);
+    Scm_QuatfMulv(r, s, qconj);
+}
+
 /*
  * Interpolation
  */
@@ -1246,6 +1258,56 @@ void Scm_QuatfSlerp(float r[], const float p[], const float q[], float t)
  */
 
 
+/*
+ * Vectors -> Quaternion
+ *   Return a quaternion that represents the rotation to rotate V to W.
+ *   NOTE: if V = -W, we can't determine a unique rotation.  This routine
+ *   returns #,(quatf e0 e1 e2 e3) where eN is very small number.
+ *   It's caller's responsibility to detect the case.
+ */
+void Scm_VectorsToQuatfv(float r[], const float v[], const float w[])
+{
+    float p[4], c, s2, f;
+    SCM_VECTOR4F_CROSSV(p, v, w);
+    c = SCM_VECTOR4F_DOTV(v, w);   /* cos(t) */
+    s2 = SCM_VECTOR4F_DOTV(p, p);  /* sin^2(t) */
+    if (s2 > 0) {               /* NB: should we consider epsilon? */
+        f = sqrtf((1-c) / (2*s2));     /* sin(t/2)/sin(t) sans sign */
+    } else {
+        f = 0.0f;
+    }
+    r[0] = f*p[0];
+    r[1] = f*p[1];
+    r[2] = f*p[2];
+    r[3] = sqrtf((1+c)/2);
+}
+
+/*
+ * Axes -> Quaternion
+ *
+ *  (v1, v2) and (w1,w2) are pair of perpendicular unit vectors.  Calculates
+ *  a rotation that transforms v1 to w1 and v2 to w2.
+ */
+void Scm_AxesToQuatfv(float r[],
+                      const float v1[],
+                      const float v2[],
+                      const float w1[],
+                      const float w2[])
+{
+    float q1[4], q2[4], c, t, s2, axis[4], wt[4];
+    Scm_VectorsToQuatfv(q1, v1, w1);
+    Scm_QuatfTransformv(wt, q1, v2);
+    SCM_VECTOR4F_CROSSV(axis, wt, w2);
+    SCM_VECTOR4F_NORMALIZEV(axis);
+    c = SCM_VECTOR4F_DOTV(w2, wt);  /* cos(t) */
+    if (c < -1.0f) c = -1.0f;
+    else if (c > 1.0f)  c = 1.0f;
+    t = acosf(c);
+    s2 = sinf(t/2);
+    q2[0] = axis[0] * s2; q2[1] = axis[1] * s2; q2[2] = axis[2] * s2;
+    q2[3] = cosf(t/2);
+    Scm_QuatfMulv(r, q2, q1);
+}
 
 /*=============================================================
  * Initialization
