@@ -1,7 +1,7 @@
 /*
  * gauche/math3d.h - 3D vector and matrix arithmetic 
  *
- *  Copyright(C) 2002-2003 by Shiro Kawai (shiro@acm.org)
+ *  Copyright (c) 2002-2008  Shiro Kawai  <shiro@acm.org>
  *
  *  Permission to use, copy, modify, distribute this software and
  *  accompanying documentation for any purpose is hereby granted,
@@ -12,7 +12,7 @@
  *  warranty.  In no circumstances the author(s) shall be liable
  *  for any damages arising out of the use of this software.
  *
- *  $Id: math3d.h,v 1.21 2006/11/09 19:46:37 shirok Exp $
+ *  $Id: math3d.h,v 1.24 2008-06-05 11:20:54 shirok Exp $
  */
 
 /* Vector and matrix arithmetics, specialized for 3D graphics calculation. */
@@ -275,6 +275,14 @@ SCM_CLASS_DECL(Scm_QuatfClass);
         }                                               \
     } while (0)
 
+#define SCM_QUATF_CONJUGATEV(q, p)              \
+    do {                                        \
+        q[0] = -p[0];                           \
+        q[1] = -p[1];                           \
+        q[2] = -p[2];                           \
+        q[3] = p[3];                            \
+    } while (0)
+
 extern ScmObj Scm_MakeQuatf(float x, float y, float z, float w);
 extern ScmObj Scm_MakeQuatfv(const float d[4]);
 extern ScmObj Scm_MakeQuatfvShared(float d[4]);
@@ -289,7 +297,6 @@ extern void   Scm_QuatfSubv(float *r, const float *p, const float *q);
 extern ScmObj Scm_QuatfMul(const ScmQuatf *p, const ScmQuatf *q);
 extern void   Scm_QuatfMulv(float *r, const float *p, const float *q);
 extern ScmObj Scm_QuatfNormalize(const ScmQuatf *q);
-extern ScmObj Scm_QuatfNormalizev(float *q);
 extern ScmObj Scm_QuatfNormalizeX(ScmQuatf *q);
 
 /* q[] must be a unit quaternion */
@@ -298,9 +305,20 @@ extern void   Scm_QuatfToMatrix4fv(float *m, const float *q);
 /* m[] must be an orthogonal matrix */
 extern void   Scm_Matrix4fToQuatfv(float *q, const float *m);
 
+/* q[] must be a unit quaternion */
+extern void Scm_QuatfTransformv(float r[], const float q[], const float v[]);
+
 /* p[] and q[] must be unit quaternions */
 extern void   Scm_QuatfSlerp(float *r, const float *p, const float *q, float t);
 
+/* v[], v1[], v2[], w[], w1[] and w2[] must be unit vectors */
+extern void   Scm_VectorsToQuatfv(float *r, const float *v, const float *w);
+extern void   Scm_AxesToQuatfv(float r[],
+                               const float v1[],
+                               const float v2[],
+                               const float w1[],
+                               const float w2[]);
+
 /*=============================================================
  * Matrix
  */
